const cliInfo = {
  contact: "everyone@disroot.org",
  repository: "https://git.disroot.org/everyone/AoC/src/branch/cli",
}

import { ArgumentValue, Command } from "https://deno.land/x/cliffy@v1.0.0-rc.3/command/mod.ts"

const { load } = await import("https://deno.land/std@0.204.0/dotenv/mod.ts")
await load({
  export: true,
})

const sleep = (ms: number) => new Promise((resolve) => setTimeout(resolve, ms))

const error = (msg: string) => {
  console.log(msg)
  Deno.exit(1)
}
const consoleUpdate = async (msg: string) =>
  await Deno.stdout.write(new TextEncoder().encode("\r" + msg))

const EventDate = ({ value }: ArgumentValue) => {
  const results = value.split("/").map((date, _, arr) => {
    if (arr.length <= 1) {
      error(`${value} is not a valid event\nEvent must be in format 'year/day'`)
    }
    const parsedDate = Number(date)
    if (!parsedDate) error(`${date} in ${value} is not a number`)
    return parsedDate
  })
  return [value, ...results] as Event
}
type Event = [unmodified: string, year: number, day: number]
const aoc = async (
  init:
    | { type: "input"; date: Event }
    | { type: "answer"; date: Event; answer: unknown; level: 1 | 2 },
): Promise<string> => {
  const timestamp = localStorage.getItem("timestamp")
  if (timestamp && Number(timestamp) > new Date().getTime()) {
    await consoleUpdate(
      `Woah, slow down. waiting for ${(Number(timestamp) - new Date().getTime()) / 1000} seconds`,
    )
    await sleep(1000)
    return aoc(init)
  } else localStorage.setItem("timestamp", String(new Date().getTime() + 60000))

  const headers = new Headers({
    Cookie: Deno.env.get("cookie")!,
    "User-Agent": JSON.stringify(cliInfo),
  })

  let body: string | undefined | URLSearchParams

  if ("level" in init) {
    headers.set("Content-Type", "application/x-www-form-urlencoded")
    body = new URLSearchParams({
      level: String(init.level),
      answer: String(init.answer),
    })
  }

  const res = await self.fetch(
    `https://adventofcode.com/${init.date[1]}/day/${init.date[2]}/${init.type}`,
    { headers, body, method: init.type === "answer" ? "POST" : "GET" },
  )
  if (res.status != 200) error(res.statusText)

  const text = await res.text()
  if (init.type === "answer") {
    const { DOMParser } = await import(
      "https://deno.land/x/deno_dom@v0.1.41-alpha-artifacts/deno-dom-wasm.ts"
    )
    const dom = new DOMParser().parseFromString(text, "text/html")
    const content = dom?.getElementsByTagName("main")[0].textContent
    return content!
  }
  return text
}

const fetch = new Command<{ event: Event; cookie: string }>()
  .description("Fetches your puzzle's input text")
  .option("--force", "Redownloads the selected event's input")
  .action(async ({ force, event }) => {
    const path = `${event[0]}/input`
    try {
      if (force) throw force
      await Deno.stat(path)
      error(`input for ${event[0]} already exists`)
    } catch {
      const { ensureDir } = await import(
        "https://deno.land/std@0.204.0/fs/ensure_dir.ts"
      )
      const input = (await aoc({ type: "input", date: event })).replace(/\n$/, "")

      await ensureDir(event[0])
      await Deno.writeTextFile(path, input)

      console.log("Downloaded to", path)
    }
  })

// const Level =

const submit = new Command<{ event: Event; cookie: string }>()
  .description("To Be Implemented")
  .type("Level", ({ value }: ArgumentValue) => {
    const parsedLevel = Number(value)
    if (!parsedLevel) error(`level requires to be a number`)
    if (parsedLevel >= 3) error(`level can't be more then 2`)
    return parsedLevel as 1 | 2
  })
  .option("--level <number:Level>", "", { required: true })
  .option("--answer <string>", "", { required: true })
  .action(async ({ level, answer, event }) => {
    const res = await aoc({ type: "answer", date: event, answer, level })
    console.log(res)
  })

new Command()
  .name("aoc-cli")
  .help({ hints: true })
  .globalEnv(
    "cookie=<string>",
    "Your AdventofCode.com authentication cookie.\nCan be set with a .env file",
    { required: true },
  )
  .type("Event", EventDate)
  .globalOption("--event <string:Event>", "", { required: true })
  .action((opts) => opts)
  .command("fetch", fetch)
  .command("submit", submit)
  .parse()
